@@ -1,22 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
-  classic_pi - Classic Priority Inversion deadlock test case
-  
-   Copyright (C) 2006, 2007 Clark Williams <williams@redhat.com>
-  
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
-   USA */
+ * classic_pi - Classic Priority Inversion deadlock test case
+ * Copyright (C) 2006, 2007 Clark Williams <williams@redhat.com>
+ */
 
 /* This program tests Priority Inheritance mutexes and their ability
    to avoid Priority Inversion deadlocks
